import { StrictMode } from 'react';
import { createRoot } from 'react-dom/client';
import App from './App.tsx';
import './index.css';
import { ImageProvider } from './lib/context/ImageContext.tsx';

// Script to detect and handle Android startup issues
function setupAndroidSupport() {
  // Detect Android Chrome specifically
  const isAndroidChrome = /Android/.test(navigator.userAgent) && /Chrome\//.test(navigator.userAgent);
  
  if (isAndroidChrome) {
    // Add Android classes for special styling
    document.documentElement.classList.add('chrome-android-pwa');
    document.body.classList.add('chrome-android-pwa');
    
    // Fix for layout issues on startup
    window.addEventListener('load', function() {
      // Force layout recalculation to fix visual issues
      document.body.style.transform = 'translateZ(0)';
      
      // Slight delay to ensure UI has rendered
      setTimeout(function() {
        window.scrollTo(0, 1);
      }, 100);
    });
    
    // Add Android-specific error handlers
    window.addEventListener('error', function(event) {
      console.warn('Global error caught:', event.message);
      
      // If we detect auth-related or network errors, try to recover
      if (event.message && (
          event.message.includes('auth') || 
          event.message.includes('network') ||
          event.message.includes('fetch') ||
          event.message.includes('storage'))) {
        
        console.warn('Attempting recovery from error...');
        
        // Force a UI state refresh without full page reload
        const root = document.getElementById('root');
        if (root) {
          root.style.opacity = '0.9';
          setTimeout(() => {
            root.style.opacity = '1';
          }, 100);
        }
      }
    });
    
    // Add unhandled promise rejection handler
    window.addEventListener('unhandledrejection', function(event) {
      console.warn('Unhandled promise rejection:', event.reason);
      
      // If this is a networking related error, try to recover
      if (event.reason && (
          (typeof event.reason.message === 'string' && 
           (event.reason.message.includes('network') || 
            event.reason.message.includes('fetch') ||
            event.reason.message.includes('timeout'))) ||
          event.reason === 'Network Error' ||
          event.reason === 'Failed to fetch')) {
        
        console.warn('Network related error detected, checking online status');
        
        // Check if we're online
        if (!navigator.onLine) {
          console.warn('Device is offline, waiting for online status');
          
          // Listen for when we come back online
          const onlineHandler = function() {
            console.log('Device is back online, refreshing app state');
            window.removeEventListener('online', onlineHandler);
            
            // Try to restore app state without reload
            const root = document.getElementById('root');
            if (root) {
              root.style.opacity = '0.95';
              setTimeout(() => {
                root.style.opacity = '1';
                
                // Try to trigger a visibility change to refresh data
                document.dispatchEvent(new Event('visibilitychange'));
              }, 100);
            }
          };
          
          window.addEventListener('online', onlineHandler);
        }
      }
    });
  }
  
  // Detect standalone mode (PWA)
  if (window.matchMedia('(display-mode: standalone)').matches || 
      (window.navigator as any).standalone || 
      document.referrer.includes('android-app://')) {
    
    document.body.classList.add('standalone-app');
    
    // For all PWAs, set up browser storage error handling
    try {
      // Test localStorage to see if it's available
      localStorage.setItem('pwa_test', '1');
      localStorage.removeItem('pwa_test');
    } catch (storageError) {
      console.warn('localStorage not available, using fallback storage');
      
      // Create in-memory fallback storage
      const memoryStorage: Record<string, string> = {};
      
      // Override localStorage methods
      Object.defineProperty(window, 'localStorage', {
        value: {
          setItem: (key: string, value: string) => {
            memoryStorage[key] = value;
          },
          getItem: (key: string) => memoryStorage[key] || null,
          removeItem: (key: string) => {
            delete memoryStorage[key];
          },
          clear: () => {
            Object.keys(memoryStorage).forEach(key => delete memoryStorage[key]);
          }
        },
        writable: false
      });
    }
    
    // For all PWAs, initialize a heartbeat to keep the app alive
    // This is especially important for Android Chrome
    setInterval(() => {
      // Keep the app responsive even when backgrounded
      console.log('App heartbeat: ' + new Date().toISOString());
      
      // Force minimal UI update to prevent engine from sleeping
      const root = document.getElementById('root');
      if (root) {
        const currentOpacity = parseFloat(getComputedStyle(root).opacity);
        root.style.opacity = (currentOpacity >= 1 ? '0.99' : '1');
      }
      
      // For offline support - check connection periodically and trigger events if needed
      try {
        // Simple fetch to check connection without actually making a network request
        fetch('https://www.google.com/favicon.ico', { 
          mode: 'no-cors',
          cache: 'no-store',
          signal: AbortSignal.timeout(3000) // Timeout after 3 seconds
        })
        .then(() => {
          if (!navigator.onLine) {
            // Browser thinks we're offline but we just successfully fetched
            console.log('Connection detected but navigator.onLine is false, fixing...');
            
            // Manually trigger an online event
            window.dispatchEvent(new Event('online'));
          }
        })
        .catch(() => {
          if (navigator.onLine) {
            // Browser thinks we're online but fetch failed
            console.log('No connection but navigator.onLine is true, fixing...');
            
            // Manually trigger an offline event
            window.dispatchEvent(new Event('offline'));
          }
        });
      } catch (err) {
        // Ignore errors in connection check
      }
    }, 30000);
  }

  // Create a connection error handler for the entire app
  const createConnectionMonitor = () => {
    let lastOnlineStatus = navigator.onLine;
    let pendingSyncs = 0;
    
    // Function to check if any pending syncs need to be processed
    const processPendingSyncs = () => {
      // Check for pending syncs when back online
      if (pendingSyncs > 0) {
        console.log(`Processing ${pendingSyncs} pending syncs`);
        
        // Dispatch a custom event that our sync handlers can listen for
        window.dispatchEvent(new CustomEvent('fittrack:sync'));
        
        pendingSyncs = 0;
      }
    };
    
    // Check connection status periodically
    setInterval(() => {
      const isOnline = navigator.onLine;
      
      // If status changed from offline to online
      if (!lastOnlineStatus && isOnline) {
        console.log('Connection restored, processing pending syncs');
        processPendingSyncs();
      } 
      // If status changed from online to offline
      else if (lastOnlineStatus && !isOnline) {
        console.log('Connection lost, entering offline mode');
      }
      
      lastOnlineStatus = isOnline;
    }, 5000);
    
    // Listen for online/offline events
    window.addEventListener('online', () => {
      console.log('Browser reports online status');
      processPendingSyncs();
    });
    
    window.addEventListener('offline', () => {
      console.log('Browser reports offline status');
      pendingSyncs++; // Increment sync counter when going offline
    });
    
    // Listen for custom sync requests from the app
    window.addEventListener('fittrack:requestSync', () => {
      if (navigator.onLine) {
        processPendingSyncs();
      } else {
        pendingSyncs++;
      }
    });
  };
  
  // Initialize connection monitor
  createConnectionMonitor();
}

// Run Android support setup
setupAndroidSupport();

// Standard React initialization
createRoot(document.getElementById('root')!).render(
  <StrictMode>
    <ImageProvider>
      <App />
    </ImageProvider>
  </StrictMode>
);